@@ -86,8 +86,7 @@ class Model(object):
         default_formats
         if nodeClass in default_formats:
             return deepcopy(default_formats[nodeClass])
-        else:
-            return None
+        return None
 
     def getCurrentModelPath(self):
         if self.existNode('current_path'):
@@ -229,12 +228,11 @@ class Model(object):
         return res
 
     def isNodeInScenario(self, nodeId):
-        if nodeId in self._scenarioDic:
-            return True
-        else:
-            return False
+        return nodeId in self._scenarioDic
 
-    def evaluateNode(self, nodeId, dims=None, rows=None, columns=None, summaryBy='sum', bottomTotal=False, rightTotal=False, fromRow=0, toRow=0, hideEmpty=None):
+    def evaluateNode(self, nodeId, dims=None, rows=None, columns=None, summaryBy='sum',
+                     bottomTotal=False, rightTotal=False, fromRow=0, toRow=0, hideEmpty=None,
+                     rowOrder='original', columnOrder='original'):
         """Evaluate node. Call evaluator class for implement diferent evaluators."""
         if self.existNode(nodeId):
             result = None
@@ -246,7 +244,9 @@ class Model(object):
             if not result is None:
                 self.evaluationVersion += 1
                 evaluator = Evaluator.createInstance(result)
-                return evaluator.evaluateNode(result, self.nodeDic, nodeId, dims, rows, columns, summaryBy, bottomTotal, rightTotal, fromRow, toRow, hideEmpty)
+                return evaluator.evaluateNode(result, self.nodeDic, nodeId, dims, rows, columns,
+                                              summaryBy, bottomTotal, rightTotal, fromRow, toRow,
+                                              hideEmpty, rowOrder, columnOrder)
         return ''
 
     def executeButton(self, nodeId):
@@ -263,7 +263,6 @@ class Model(object):
     def previewNode(self, nodeId, debugMode=""):
         """Perform preview of a node"""
         try:
-
             result = None
             if self.existNode(nodeId):
                 if debugMode:
@@ -298,7 +297,6 @@ class Model(object):
         """Evaluate node. Used for pivotgrid"""
         nodeId = query['cube']
         if self.existNode(nodeId):
-
             result = None
             if nodeId in self._scenarioDic:
                 result = self._scenarioDic[nodeId]
@@ -366,7 +364,8 @@ class Model(object):
         nodeList = self.findNodes('moduleId', moduleId)
         nodeList.sort(key=lambda x: int(x.z))
         for node in nodeList:
-            exceptions = ['definition'] if node.nodeClass == 'text' else ['definition', 'description']
+            exceptions = ['definition'] if node.nodeClass == 'text' else [
+                'definition', 'description']
             res['nodes'].append(node.toObj(
                 exceptions=exceptions, fillDefaultProperties=True))
         return res
@@ -426,6 +425,11 @@ class Model(object):
                 if new_definition:
                     node.definition = new_definition
 
+    def getSelectorValue(self, nodeId):
+        """Return selector value if node is of type selector"""
+        if self.isSelector(nodeId):
+            return self.getNode(nodeId).result.value
+
     def release(self):
         """Release model. Free all resources """
         if not self._modelNode is None:
@@ -481,8 +485,7 @@ class Model(object):
                 node._definition = 'result = ' + str(newNodeId)
             del self.nodeDic[oldNodeId]
             return True
-        else:
-            return False
+        return False
 
     def setNodeProperties(self, nodeId, properties):
         """Update properties of a node"""
@@ -1340,14 +1343,14 @@ class Model(object):
             if not os.path.isdir(user_lib_path):
                 os.makedirs(user_lib_path, exist_ok=True)
             # copy base venv folders
-            os.system(f'cp -r -u {venv_path}-bkp/* {user_lib_path}')
+            os.system(f'cp -r -u {venv_path}-bkp/* "{user_lib_path}"')
         except:
             pass
 
         # create symlink from user /public site-package
         if os.path.isdir(user_lib_path):
             os.system(f'rm -rf {venv_path}')
-            os.system(f'ln -s -f {user_lib_path} {venv_path}')
+            os.system(f'ln -s -f "{user_lib_path}" {venv_path}')
 
     def applyBackwardCompatibility(self):
         # update old selector definition
